/* Revised 5/6/02 William Hays - CTI */

#include "libcell.h"

/*  Note:

An alias can only reference to tags either within it's own program or Controller Scoped tags.
An alias can not reference to tags within other programs.  This makes alias checking 
quite a bit easier as we only need to check two places...   

This routine will search for alias references.  If one is found, the alias tag ID data will be
rewrote to point to the actual data object.  The rest of the data remains untouched.  Hopefully
this will work correctly....

*/


/* Three possible return values

   0 = Success, alias found
   -1  Error
   1 = Tag not found
*/

int aliascheck(_tag_detail * tag, _tag_data * proglist, _tag_data * mainlist,
		int debug)
{
  int y;

  if (tag == NULL)
  {
    CELLERROR(1,"Tag structure not allocated");
    return -1;
  }

  if (tag->type >> 12 == 1)
  {
    CELLERROR(2,"Tag type incorrect");
    return -1;
  }

  if (((tag->topbase>>24) & 0xf) != 0x4)
    {
    return 1; /* if the forth byte of the topbase is 0x04, then the tag is an alias.  */
    }

  if (((tag->topbase>>16) & 1) == 1)		/* Is the tag Controller Scoped? */
  {
      for (y = 0; y < mainlist->count; y++)
      {
	if (mainlist->tag[y]->base == (tag->topbase & 0xffff))
	{
	  dprint(DEBUG_VALUES, "%s aliases to %s - %08lX\n", tag->name,
		 mainlist->tag[y]->name, mainlist->tag[y]->base);
	  tag->alias_size = mainlist->tag[y]->size;
	  tag->alias_linkid = mainlist->tag[y]->linkid;
	  tag->alias_type = mainlist->tag[y]->type;
	  tag->alias_topbase = mainlist->tag[y]->topbase;
	  tag->alias_base = mainlist->tag[y]->base;
	  return 0;  /* alias found */
	}
      }
    
    {
      char ac_dmesg[80];

      sprintf(ac_dmesg,"WARN: Could not find alias tag %s - %08lX\n", tag->name,
	   tag->topbase);
      CELLERROR(3,ac_dmesg);
    }

    return -1;
  }

  if (proglist == NULL)
  {
    CELLERROR(4,"Proglist structure is null");
    return -1;
  }

  if (((tag->topbase>>16) & 2) == 2)		/* We are checking a program scoped tag here against the other program tags */
  {

    dprint(DEBUG_VALUES, "Searching for alias to tag %s    ", tag->name);
    if (tag->id != (tag->topbase & 0xffff))
    {
      for (y = 0; y < proglist->count; y++)
      {
	if (proglist->tag[y]->id == (tag->topbase & 0xffff))
	{
	  dprint(DEBUG_VALUES, "%s aliases to %s - %08lX\n", tag->name,
		 proglist->tag[y]->name, proglist->tag[y]->id);
	  tag->alias_size = proglist->tag[y]->size;
	  tag->alias_linkid = proglist->tag[y]->linkid;
	  tag->alias_type = proglist->tag[y]->type;
	  tag->alias_topbase = proglist->tag[y]->topbase;
	  tag->alias_id = proglist->tag[y]->id;
	  return 0;
	}
      }
      /* We didn't find a match in the program scoped tags, search controller scoped tags now... */
      dprint(DEBUG_VALUES, "Searching Controller tags...   ");
      for (y = 0; y < mainlist->count; y++)
      {
	if (mainlist->tag[y]->base == (tag->topbase & 0xffff))
	{
	  dprint(DEBUG_VALUES,
		 "%s aliases to controller scoped tag %s - %08lX\n",
		 tag->name, mainlist->tag[y]->name, mainlist->tag[y]->base);
	  tag->alias_size = mainlist->tag[y]->size;
	  tag->alias_linkid = mainlist->tag[y]->linkid;
	  tag->alias_type = mainlist->tag[y]->type;
	  tag->alias_topbase = mainlist->tag[y]->topbase;
	  tag->alias_base = mainlist->tag[y]->base;
	  tag->alias_id = mainlist->tag[y]->id;
	  return 0;
	}
      }
      dprint(DEBUG_VALUES, "Nothing found.\n");
      return 1;
    }
  }
  CELLERROR(9,"tag->id was not zero structure is null");
  return -1;			/* If we get here, we have problems...  */
}
